<?php

namespace Radar\Exceptions;

/**
 * Description of SurveyException
 */
class SurveyException extends \Exception
{

    public $originalTrace = [];
    public $filteredException = [];
    public $errorMessage;

    /**
     * Показывать ли кастомное сообщение при вброшенном исключении
     * @var bool
     */
    public $showFilteredMessage = true;

    public function __construct($message = false, $trace = false)
    {
        parent::__construct();

        if (!$trace) {
            $this->originalTrace = $this->getTrace()[0];
        } else {
            $this->originalTrace = $trace[0];
        }

        if (!$message) {
            $this->errorMessage = $this->getMessage();
        } else {
            $this->errorMessage = $message;
        }

        if (empty($this->errorMessage)) {
            throw new self('Message string must not be empty !');
        }

        if (empty($this->originalTrace)) {
            throw new self('Trace array must not be empty !');
        }

        if (!is_array($this->originalTrace)) {
            throw new self('Trace must be an array !');
        }

//  Отобразим trace без полного пути к файлу
        printf(
                "<html>
                    <head>
                        <link rel='stylesheet' href='https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css'>
                    </head>
                    <body>
                        <div class='row'>
                            <div class='col-xs-8 col-xs-offset-2'>
                                <h3 class=text-center>Error occured :( </h3>
                                <pre><code>%s</code></pre>
                            </div>
                        </div>
                    </body>
                </html>", print_r(
                        $this->filterOriginalException($this->originalTrace, $this->errorMessage), true
                )
        );

        if (php_sapi_name() != 'cli') {
//  И остановим выполнение скрипта, чтобы не отобразить оригинальный stack trace
            $this->setShowCustomExceptioMessage(true);
            die($this->errorMessage);
        }
    }

    public function getFilteredTrace()
    {
        return $this->filteredException;
    }

    public function getOriginalTrace()
    {
        return $this->originalTrace;
    }

    public function getOriginalMessage()
    {
        return $this->errorMessage;
    }

    public function setShowCustomExceptioMessage($showFilteredMessage)
    {
        $this->showFilteredMessage = (bool) $showFilteredMessage;
    }

    public function getShowCustomExceptionMessage()
    {
        return $this->showFilteredMessage;
    }

    private function filterOriginalException(array $trace, $message)
    {
        $explodedPathToTracedFile = explode('/', $trace['file']);
        $trace['message'] = $message;
        $trace['file'] = $explodedPathToTracedFile[count($explodedPathToTracedFile) - 1];

        return $trace;
    }

}
