<?php

namespace Radar\Mailer;

/**
 * Простая обертка над PHPMailer
 *
 * @author Zelepukin
 */
class Mailer
{

    /**
     * @var \PHPMailer 
     */
    private $phpMailer;

    /**
     * Использовать ли HTML разметку в письме
     * @var boolean 
     */
    private $useHTML = false;

    public function __construct()
    {
        require 'vendor/phpmailer/phpmailer/PHPMailerAutoload.php';
        $this->phpMailer = new \PHPMailer;
    }

    /**
     * Отправляет электронное письмо указанным адресатам
     * @param string|array $emails Список адресатов
     * @param string $messageBody Тело сообщения
     * @param string $pathToAttachment Путь к прикрепленному файлу
     * @return boolean
     */
    public function send($emails, $messageBody, $pathToAttachment = false)
    {
        $arAddresses = $this->createAddressesFromFormat($emails);

        $this->phpMailer->setFrom('info@radar-survey.ru', 'RadarNumbersParser');

        foreach ($arAddresses as $address) {
            $this->phpMailer->addAddress($address);
        }

        if ($pathToAttachment) {
            $this->phpMailer->addAttachment($pathToAttachment);
        }

        $this->phpMailer->Subject = 'Файл с информацией по номерам - ' . date('Y-m-d H:i:s');
        $this->phpMailer->Body = $messageBody;

        $this->phpMailer->isHTML($this->useHTML);
        $this->phpMailer->CharSet = 'UTF-8';

        return $this->phpMailer->send();
    }

    /**
     * Сеттер для переменной использования html-разметки в сообщении
     * @param boolean $useHTML
     * @throws \Radar\Exceptions\InvalidArgumentException
     */
    public function setUseHTMLInMessageBody($useHTML)
    {
        if (!is_bool($useHTML)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Var must contain boolean value !');
        }

        $this->useHTML = $useHTML;
    }

    /**
     * Геттер для переменной использования html-разметки в сообщении
     * @return boolean
     */
    public function isUsingHTMLInMessageBody()
    {
        return $this->useHTML;
    }

    /**
     * Проверяет правильность формата адреса электронной почты
     * @param string $email
     * @return boolean|string
     */
    public function validateAddressFormat($email)
    {
        return filter_var($email, FILTER_VALIDATE_EMAIL);
    }

    /**
     * Создает массив адресов из строки или возвращает ранее переданный массив
     * @param string|array $emails
     * @return array
     */
    private function createAddressesFromFormat($emails)
    {
        if (is_string($emails)) {
            return $this->createAddressesFromString($emails);
        } elseif (is_array($emails)) {
            return $emails;
        }
    }

    /**
     * Создает массив адресов из строки по пробелу,табуляции,запятой,точке с запятой
     * @param string $emails строка с электронными адресами
     * @return array
     */
    private function createAddressesFromString($emails)
    {
        return preg_split('#[\s,;]#', $emails);
    }

}
