<?php

namespace Radar\NumbersParser;

/**
 * Description of CsvParser
 *
 * @author Zelepukin
 */
class CsvParser
{

    /**
     * Файлы с каким расширением нам нужны
     * @var string
     */
    private $neededFilesExtension;

    /**
     * Исходная кодировка данныхс сайта
     * @var string 
     */
    private $sourceEncoding;

    /**
     * 
     * @param string $neededFileExtension
     * @param string $sourceEncoding
     */
    public function __construct($neededFileExtension, $sourceEncoding = 'Windows-1251')
    {
        $this->neededFilesExtension = $neededFileExtension;
        $this->sourceEncoding = $sourceEncoding;
    }

    /**
     * Парсит файл csv
     * @param string $pathToFile Путь к файлу
     * @return array вида строка=>[колонки]
     */
    public function parseCsvFile($pathToFile)
    {
        if (!$file = fopen($pathToFile, 'rb')) {
            echo 'Cannot open ' . $pathToFile . ' file for reading !';
            return false;
        }

        if (System::checkConsoleArg('-v')) {
            echo "Parsing {$pathToFile} file !\n";
        }

        $arCsvData = [];
        //  Пропускаем заголовки, сдвигая указатель на одну строку
        fgetcsv($file, 0, ';');

        $i = 0;
        while ($row = fgetcsv($file, 0, ';')) {
            foreach ($row as $index => $column) {
                if (empty($column)) {
                    continue;
                }
                $arCsvData[$i][$index] = trim(mb_convert_encoding($column, 'UTF-8', $this->sourceEncoding));
            }
            $i++;
        }

        fclose($file);

        return $arCsvData;
    }

    /**
     * Возвращает список файлов и вложенных директорий в указанной директории :)
     * @param string $parentDirName
     * @return array
     */
    public function getFilesDirectoriesList($parentDirName)
    {
        $dirs = scandir($parentDirName);

        $arExcludeDirs = ['.', '..'];
        $arDirsList = [];

        foreach ($dirs as $dir) {
            if (!in_array($dir, $arExcludeDirs)) {
                $arDirsList[] = $dir;
            }
        }

        return $arDirsList;
    }

    /**
     * Возвращает путь к новейшей директории с файлами 
     * @param string $parentDirName Путь к родительской директории
     * @return string
     */
    public function getNewestDir($parentDirName)
    {
        $arDirs = $this->getFilesDirectoriesList($parentDirName);

        $arConvertedDirs = [];

        foreach ($arDirs as $date) {
            $arConvertedDirs[] = strtotime($date);
        }

        arsort($arConvertedDirs);

        reset($arConvertedDirs);

        $newestDirName = date('d.m.Y', current($arConvertedDirs));

        if (System::checkConsoleArg('-v')) {
            echo "The newest dir date is {$newestDirName}\n";
        }

        return $newestDirName;
    }

    /**
     * Возвращает список файлов из директории
     * @param string $parentDirName Путь к директории с файлами
     * @return boolean|array
     */
    public function getFilesList($parentDirName)
    {
        $arFiles = [];

        if (!$dirContent = scandir($parentDirName)) {
            echo 'Error ! Cannot get content list from ' . $parentDirName;
            return false;
        }

        foreach ($dirContent as $content) {
            $fullPathToFile = $parentDirName . '/' . $content;
            if (is_file($fullPathToFile) && pathinfo($fullPathToFile)['extension'] == $this->neededFilesExtension) {
                $arFiles[] = $parentDirName . '/' . $content;
            }
        }

        return $arFiles;
    }

    /**
     * Создает csv-файл
     * @param string $pathTofile Путь где будет создан\перезаписан файл
     * @param array $head Названия колонок 
     * @param array $data Данные для записи
     * @param boolean $transposeData Нужно ли траспонировать строки в массиве с данными
     * @return boolean
     * @throws \Exception Если не удалось открыть файл для записи
     */
    public function createCsvFile($pathTofile, array $head, array $data, $transposeData = false)
    {
        $arDataToWrite = [];

//      Если нужно, транспонируем колонки в строки в массиве с данными
        if ($transposeData) {
            foreach ($data as $row => $columns) {
                foreach ($columns as $index => $column) {
                    $arDataToWrite[$index][$row] = $column;
                }
            }
        } else {
            $arDataToWrite = $data;
        }

//  Если нет директории - создаем
        $dirName = dirname($pathTofile);
        if (!is_dir($dirName)) {
            mkdir($dirName, 0777, true);
        }

//      Открываем/создаем файл для записи 
        if ($file = fopen($pathTofile, 'wb')) {
//          Сначала пишем заголовки/названия колонок
            fputcsv($file, $head, ';');
//          Потом данные из массива со столбцами
            foreach ($arDataToWrite as $rowKey => $columns) {
                fputcsv($file, $columns, ';');
            }
            fclose($file);
        } else {
            throw new \Exception("Can't open {$pathTofile} for writing !");
        }

        return true;
    }

}
