<?php

namespace Radar\Users;

/**
 * Description of UserTrait
 */
trait UserTrait
{

    /**
     * Проверяет данные пользователя в базе
     * @param array $dataToCheck Данные для проверки в виде [название_колонки_в_базе => значение_колонки_в_базе]
     * @param string $customTable Если указано, проверять будет в этой таблице
     * @return boolean true если удалось выбрать строку из базы по этим условиям, false если нет
     * @throws \Radar\Exceptions\InvalidArgumentException Если массив с данными для проверки пуст
     */
    public function checkUserData(array $dataToCheck, $customTable = false)
    {
        $conditions = '';

        if (empty($dataToCheck)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Array with columns names to check must not be empty !');
        }

        if ($customTable) {
            $userDataTableName = $customTable;
        } else {
            $userDataTableName = $this->userDataTableName;
        }

        $i = 0;
        foreach ($dataToCheck as $columnName => $columnValue) {
            if ($i > 0) {
                $conditions .= " AND ";
            }

            $conditions .= "`{$columnName}`= {$this->db->dbh->quote($columnValue)} ";

            $i++;
        }

        $result = $this->db->select(['id'], [$userDataTableName], $conditions, false, true);

        if ($result->rowCount() > 0) {
            return true;
        }

        return false;
    }

    /**
     * Сеттер для переменной с названием таблицы с данными пользователя
     * @param string $userDataTableName
     * @throws \Radar\Exceptions\InvalidArgumentException Если в аргументе пусто
     */
    public function setUserDataTableName($userDataTableName)
    {
        if (empty($userDataTableName)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Variable with user data table name must not be empty !');
        }

        $this->userDataTableName = $userDataTableName;
    }

    /**
     * Геттер для переменной с названием таблицы с данными пользователя
     * @return string
     */
    public function getUserDataTableName()
    {
        return $this->userDataTableName;
    }

}
