<?php

require 'config.php';
require 'vendor/autoload.php';
require 'autoload.php';

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

// Пробуем отобразить стак-трейс слима
$configuration = [
    'settings' => [
        'displayErrorDetails' => true,
    ],
];

$container = new \Slim\Container($configuration);

$slim = new \Slim\App($container);
$db = new \Radar\DataBase\DataBase(DB_CONNECTION_STRING, DB_USER, DB_PASS);
$user = new Radar\Users\NumbersParserUser($db, USERS_DATA_TABLE);
$csvParser = new Radar\NumbersParser\CsvParser(NEEDED_FILE_EXTENSION);
$s = new Radar\NumbersParser\System($db, $csvParser, NUMBERS_DATA_TABLE_NAME, METADATA_TABLE_NAME);

// Подключаем Твиг
$container['view'] = function ($container) {
    $view = new \Slim\Views\Twig(__DIR__ . '/tpl/', [
        'cache' => 'tmp/pages_cache',
        'debug' => true
    ]);

    $view->addExtension(new \Slim\Views\TwigExtension(
                    $container['router'], $container['request']->getUri()
    ));

    return $view;
};

$container['user'] = function () use ($user) {
    return $user;
};

$container['db'] = function () use ($db) {
    return $db;
};

$container['system'] = function () use ($s) {
    return $s;
};
$container['csvParser'] = function () use ($csvParser) {
    return $csvParser;
};

$container['pagesList'] = [
    'main' => ['link' => 'main', 'name' => 'Главная страница'],
    'nttb' => ['link' => 'nttb', 'name' => 'Табличное отображение']
];

$slim->get('/', function (Request $request, Response $response, $args) {
    if (!$this->user->isLogged()) {
        return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'login');
    } else {
        return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'main');
    }
});

$slim->get('/login', function (Request $request, Response $response, $args) {
    if ($this->user->isLogged()) {
        return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'main');
    } else {
        return $this->view->render($response, 'login.html.twig', ['title' => 'Login Page', 'pageClass' => 'login']);
    }
});

$slim->post('/login', function (Request $request, Response $response, $args) {
    if ($this->user->isLogged()) {
        return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'main');
    }

    $this->user->checkLogin($request);

    return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'main');
});

$slim->get('/main', function (Request $request, Response $response, $args) {
    if (!$this->user->isLogged()) {
        return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'login');
    } else {
        return $this->view->render($response, 'main.html.twig', ['pagesList' => $this->pagesList, 'title' => 'Main', 'pageClass' => 'main']);
    }
});

$slim->get('/nttb', function (Request $request, Response $response, $args) {
    if (!$this->user->isLogged()) {
        return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'login');
    }

    return $this->view->render($response, 'nttb.html.twig', ['pagesList' => $this->pagesList, 'title' => 'Numbers to table', 'pageClass' => 'nttb']);
});

$slim->get('/exit', function (Request $request, Response $response, $args) {
    session_destroy();
    return $response->withStatus(301)->withHeader('Location', PROJECT_LOCATION . 'login');
});

$slim->post('/search-by-number', function (Request $request, Response $response, $args) {
    if (!$request->isXhr() || !$this->user->isLogged()) {
        return false;
    }

    $phoneNumber = $request->getParsedBody()['phoneNumber'];

    if (!empty($phoneNumber)) {
        $keeper = new \Radar\NumbersData\NumbersDataKeeper($request, $this->db, UPLOADED_FILES_DIR);
        $numbersDataTableName = $this->system->getNumbersDataTableName();
        $numberData = $keeper->getNumberInfoFromBase($phoneNumber, $numbersDataTableName, false);

        if (!empty($numberData)) {
            return json_encode($numberData);
        } else {
            return json_encode(['error' => 'Cannot get data for this number !']);
        }
    }
});

$slim->post('/numbers-upload', function(Request $request, Response $response, $args) {
    $files = $request->getUploadedFiles();
    if (empty($files['file'])) {
        return 'Expect a file !';
    }

    if (!$this->user->isLogged()) {
        return false;
    }

    $uploadedFile = $files['file'];

    $keeper = new \Radar\NumbersData\NumbersDataKeeper($request, $this->db, UPLOADED_FILES_DIR);
    $keeper->saveUploadedFile($uploadedFile);

    $pathToSaveFile = $keeper->getPathToSavedFile();
    $numbersDataTableName = $this->system->getNumbersDataTableName();

    $arNumbersFromFile = $this->csvParser->parseCsvFile($pathToSaveFile);
    $arNumbersWithData = $keeper->getNumbersInfoFromBase($arNumbersFromFile, $numbersDataTableName);

    $fileName = $keeper->getUploadedFileName();
    $fileExtesion = $keeper->getUploadedFileExtension();
    $createdFilePath = CREATED_FILES_DIR . "{$fileName}_with_info.{$fileExtesion}";

//  Если получилось создать файл - сохраняем путь к нему в сессии  
    if ($this->csvParser->createCsvFile($createdFilePath, ['DexId', 'Phone'], $arNumbersWithData)) {
        $_SESSION['user']['pathToFileWithInfo'] = $createdFilePath;
    } else {
        return false;
    }

    return json_encode(['createdFileUrl' => PROJECT_LOCATION . $createdFilePath]);
});

$slim->post('/parse-numbers-from-block', function(Request $request, Response $response, $args) {
    if (!$request->isXhr() || !$this->user->isLogged()) {
        return false;
    }

    $keeper = new \Radar\NumbersData\NumbersDataKeeper($request, $this->db, UPLOADED_FILES_DIR);
    $numbersDataTableName = $this->system->getNumbersDataTableName();

    $rawNumbersString = json_decode($request->getParsedBody()['numbers']);
    $arSplittedNumbers = preg_split('#[\s]#', $rawNumbersString);

    $arNumbersWithInfo = $keeper->getNumbersInfoFromBase($arSplittedNumbers, $numbersDataTableName, false);

//  Обнулим индексы во вложенных массивах    
    array_walk($arNumbersWithInfo, function(&$val) {
        $val = array_values($val);
    });

    $arColumnsNames = ['DexId', 'Phone'];
    /// Нужно создать временный файл, если пользователь захочет скачать отображенное в таблице в файл.
    $tmpDirPath = TEMPORARY_CREATED_FILES_DIR . $this->user->getLogin() . '/';
    //  Очищаем директорию от предыдущих файлов
    $this->system->cleanDirectory($tmpDirPath);
    //  Создаем файл
    $tmpFilePath = $tmpDirPath . 'Numbers-with-info_' . date('Y-m-d_H-i-s') . '.csv';

    //  Для корректной записи в файл нужно изменить кодировку в данных
    $arConvertedData = $keeper->encodeDataArrayAll($arNumbersWithInfo);
    $this->csvParser->createCsvFile($tmpFilePath, ['CallerId', 'Phone'], $arConvertedData);

    //  Сохраняем путь к файлу в сессию
    $_SESSION['user']['pathToFileWithInfoTmp'] = $tmpFilePath;

    return $this->view->render($response, 'ajax_templates/numbers_info_table_template_ajax.html.twig', ['numbersInfo' => $arNumbersWithInfo, 'columnsNames' => $arColumnsNames, 'fileLink' => $_SESSION['user']['pathToFileWithInfoTmp']]);
});

$slim->post('/send-info-via-email', function(Request $request, Response $response, $args) {
    if (!$request->isXhr() || !$this->user->isLogged()) {
        return false;
    }

    $mailer = new \Radar\Mailer\Mailer();
    $returnMessage = false;
    $from = $request->getParsedBody()['from'];

//  Если запрос с main, то берем распарсенный файл  
    if (!empty($_SESSION['user']['pathToFileWithInfo']) && $from == 'main') {
        $pathToAttachment = $_SESSION['user']['pathToFileWithInfo'];
//  Если с nttb, то временный файл с номерами из таблицы      
    } elseif (!empty($_SESSION['user']['pathToFileWithInfoTmp']) && $from == 'nttb') {
        $pathToAttachment = $_SESSION['user']['pathToFileWithInfoTmp'];
//  Если не указан путь к файлу - возвращаем код - фронтенд выдаст сообщение об ошибке
    } else {
        $returnMessage = '1';
    }

//  Если по указанному пути нет файла - аналогично - возвращаем код
    if (!is_file($pathToAttachment)) {
        $returnMessage = '1';
    }

    if ($returnMessage) {
        return json_encode(['message' => $returnMessage]);
    }

    $address = $this->user->getLogin();
    $messageBody = "Запрошенный файл во вложении";

    if ($mailer->send($address, $messageBody, $pathToAttachment)) {
        $returnMessage = '2';
    }

    return json_encode(['message' => $returnMessage]);
});

$slim->run();
